"use strict";
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------
Object.defineProperty(exports, "__esModule", { value: true });
const node_builder_1 = require("../node-builder");
const esotope_hammerhead_1 = require("esotope-hammerhead");
const instrumented_1 = require("../instrumented");
// Transform:
// obj[prop] -->
// __get$(obj, prop)
const transformer = {
    name: 'computed-property-get',
    nodeReplacementRequireTransform: true,
    nodeTypes: esotope_hammerhead_1.Syntax.MemberExpression,
    condition: (node, parent) => {
        if (!node.computed || !parent)
            return false;
        if (node.property.type === esotope_hammerhead_1.Syntax.Literal && !(0, instrumented_1.shouldInstrumentProperty)(node.property.value))
            return false;
        // super[prop]
        if (node.object.type === esotope_hammerhead_1.Syntax.Super)
            return false;
        // object[prop] = value
        if (parent.type === esotope_hammerhead_1.Syntax.AssignmentExpression && parent.left === node)
            return false;
        // delete object[prop]
        if (parent.type === esotope_hammerhead_1.Syntax.UnaryExpression && parent.operator === 'delete')
            return false;
        // object[prop]++ || object[prop]-- || ++object[prop] || --object[prop]
        if (parent.type === esotope_hammerhead_1.Syntax.UpdateExpression && (parent.operator === '++' || parent.operator === '--'))
            return false;
        // object[prop]()
        if (parent.type === esotope_hammerhead_1.Syntax.CallExpression && parent.callee === node)
            return false;
        // new (object[prop])() || new (object[prop])
        if (parent.type === esotope_hammerhead_1.Syntax.NewExpression && parent.callee === node)
            return false;
        // for(object[prop] in source)
        if (parent.type === esotope_hammerhead_1.Syntax.ForInStatement && parent.left === node)
            return false;
        return true;
    },
    run: node => (0, node_builder_1.createComputedPropertyGetWrapper)(node.property, node.object, node.optional),
};
exports.default = transformer;module.exports = exports.default;

